@@ -737,7 +737,7 @@ get_discrete_val <- function(inters,discrete_x){
     for(j in 1:size(inters,2)){
       if((inters[i,j]==1)){
         discrete_val=discrete_x[[i,j]]
-        return
+        return( discrete_val ) 
       }
     }
   }
